@@ -409,7 +409,7 @@ intelDestroyBuffer(__DRIdrawablePrivate * driDrawPriv)
       }
    }
 
-   _mesa_reference_framebuffer((GLframebuffer **)(&(driDrawPriv->driverPrivate)), NULL);
+   _mesa_unreference_framebuffer((GLframebuffer **)(&(driDrawPriv->driverPrivate)));
 }
 
 
